;============================================================================;
;                                keyboard.asm                                ;

;============================================================================;
;get_key                                                                     ;
;    Input     : void                                                        ;
;    Output    : d6.w = key code                                             ;
;    Destroyed : void                                                        ;

; This routine is %100 compatible with idle_loop
; exept for [alpha|shift] + [direction key] which
; returns the code of the direction key without alpha

get_key
         movem.w    d0-d5/d7,-(a7)                   ;
         move.l     a0,-(a7)                         ; save registers

         btst.b     #0,apd\shut_down_if_idle(pc)     ; apd time expired ?
         beq.s      get_key\apd\continue
         bsr        get_key\wait_until_[on]_unpressed
         trap       #4
get_key\apd\continue

get_key\test_shift/diamond+[on]
         btst.b     #1,$60001A
         bne.s      get_key\test_indicators
         move.b     indicators_value(pc),d1
         and.b      #diamond_value+second_value,d1
         tst.b      d1                          ; shift or diamond set ?
         beq.s      get_key\test_indicators
         eor.b      d1,indicators_value
         bsr        refresh_indicators
         bsr        get_key\wait_until_[on]_unpressed
         cmp.b      #diamond_value+second_value,d1
         beq.s      get_key\lock_calc           ; shift and diamond set ?
get_key\turn_off_calc
         trap       #4
         bra.s      get_key\test_indicators
get_key\lock_calc
         bsr        lock_calc

  ;================================================================;
  ; Indicators format :                                            ;
  ;                                                                ;
  ; +-----------+------------+-----------+-----------+-----------+ ;
  ; |   TI-89   |    alpha   |  diamond  |   shift   |   second  | ;
  ; +-----------+------------+-----------+-----------+-----------+ ;
  ; |   TI-92   | hand (lock)|   shift   |  diamond  |   second  | ;
  ; +-----------+------------+-----------+-----------+-----------+ ;
  ; |    bit    |      3     |     2     |     1     |     0     | ;
  ; +-----------+------------+-----------+-----------+-----------+ ;
  ; |   value   |      8     |     4     |     2     |     1     | ;
  ; +-----------+------------+-----------+-----------+-----------+ ;
  ;================================================================;

get_key\test_indicators
         move.w     #%1111111111111110,d5
         bsr        get_mat

         move.b     old_indicators(pc),d1
         move.b     d6,d3

 ifd ti89
         lsr.b      #4,d3
 endc

         move.b     d3,old_indicators
         not.b      d3
         and.b      d1,d3
         eor.b      d3,indicators_value         ; xor if frontedge
         tst.b      d3
         beq.s      get_key\test_keys
         bsr        refresh_indicators

get_key\test_keys
         moveq.b    #-1,d3
         moveq.w    #keyboard\number_rows,d1    ; to test 7 rows on ti89
         moveq.w    #3,d2
         bra.s      get_key\test_keys\row_loop
get_key\test_keys\line_loop
         bsr        get_mat
         moveq.w    #7,d2                       ; to test 8 columns
get_key\test_keys\row_loop

 ifd ti89
         btst.b     d2,d6
 endc

 ifd ti92plus
         moveq.l    #7,d0
         sub.b      d2,d0
         btst.b     d0,d6
 endc

         bne.s      get_key\test_keys\row_loop\dbf
         cmp.b      #-1,d3
         bne.s      get_key\double_key
         move.b     d2,d3
         move.b     d1,d4
get_key\test_keys\row_loop\dbf
         dbf.s      d2,get_key\test_keys\row_loop
         rol.w      #1,d5
         dbf.s      d1,get_key\test_keys\line_loop
         cmp.b      #-1,d3                           ; key pressed ?
         bne.s      get_key\key_pressed

get_key\double_key
get_key\no_key_pressed
         btst.b     #0,indicators_clear_flag
         beq.s      get_key\indicators_unchanged
         move.b     old_indicators,d1
         and.b      #7,d1
         cmp.b      #7,d1
         bne.s      get_key\indicators_unchanged
         and.b      #8,indicators_value
         bsr        refresh_indicators
         clr.b      indicators_clear_flag

get_key\indicators_unchanged
         clr.w      old_key

get_key\clr_d6_and_exit
         clr.w      d6
         bra        get_key\exit

get_key\key_pressed
         moveq.w    #keyboard\number_rows,d1
         sub.b      d4,d1
         lsl.b      #3,d1

 ifd ti92plus
         move.b     d3,d2
         move.b     #7,d3
         sub.b      d2,d3
 endc

         add.b      d1,d3
         and.w      #$FF,d3
         mulu       #10,d3
         clr.l      d2

         move.b     indicators_value(pc),d1
         and.b      #diamond_value+second_value,d1
         cmp.b      #diamond_value+second_value,d1
         beq.s      get_key\double_indicator
         move.b     indicators_value(pc),d1

get_key\get_indicator\alpha_or_hand

         and.b      indicators_mask,d1
 ifd ti89
         btst.b     #alpha_bit,d1
         beq.s      get_key\get_indicator\shift
         moveq.b    #(alpha_bit+1)*2,d2
 endc

 ifd ti92plus
         btst.b     #hand_bit,d1
         beq.s      get_key\get_indicator\shift
         moveq.b    #(hand_bit+1)*2,d2
 endc

get_key\get_indicator\shift
         btst.b     #shift_bit,d1
         beq.s      get_key\get_indicator\diamond
         moveq.b    #(shift_bit+1)*2,d2

get_key\get_indicator\diamond
         btst.b     #diamond_bit,d1
         beq.s      get_key\get_indicator\second
         moveq.b    #(diamond_bit+1)*2,d2

get_key\get_indicator\second
         btst.b     #second_bit,d1
         beq.s      get_key\read_key_value_matrix
         moveq.b    #(second_bit+1)*2,d2

get_key\double_indicator
get_key\read_key_value_matrix

 ifd ti89
         lea        ti89\key_value_matrix(pc),a0
 endc

 ifd ti92plus
         lea        ti92\key_value_matrix(pc),a0
 endc

         add.w      d3,d2
         move.w     0(a0,d2.l),d6

         clr.l      apd\calls_5                 ; number of autoint 5 calls
         clr.b      apd\shut_down_if_idle       ; set to one if it's apd time

         cmp.w      old_key(pc),d6              ; has key changed ?
         bne.s      get_key\key_changed         ; yes

         btst.b     #0,in_repetition_flag(pc)   ; first delay expired flag ?
         bne.s      get_key\delay               ; yes

         cmp.l      #10,timer_20                ; no, first delay expired ?
         ble.w      get_key\clr_d6_and_exit     ; no

get_key\delay\init                              ; yes
         move.b     #1,in_repetition_flag
         clr.l      timer_20
         bra.s      get_key\no_delay

get_key\delay
         cmp.l      #2,timer_20
         ble.w      get_key\clr_d6_and_exit
         clr.l      timer_20
         bra.s      get_key\no_delay

get_key\key_changed
         clr.l      timer_20
         clr.b      in_repetition_flag

get_key\no_delay
         move.b     #1,indicators_clear_flag

         move.w     d6,old_key

get_key\test_contrast_up
         cmp.w      #8235,d6
         bne.s      get_key\test_contrast_up\end
         jsr        tios::OSContrastUp
         clr.w      d6
get_key\test_contrast_up\end

get_key\test_contrast_down
         cmp.w      #8237,d6
         bne.s      get_key\test_contrast_down\end
         jsr        tios::OSContrastDn
         clr.w      d6
get_key\test_contrast_down\end

get_key\exit
         move.l     (a7)+,a0
         movem.w    (a7)+,d0-d5/d7
         rts

;============================================================================;
;get_mat                                                                     ;
;    Input     : d5.w = row mask                                               ;
;    Output    : d6 = row(s) value                                           ;
;    Destroyed : void                                                          ;

get_mat
         move.w     d0,-(a7)
         moveq.w    #keyboard\port_delay,d0
get_mat\wait
         move.w     d5,($600018)
         dbf.s      d0,get_mat\wait
         move.w     (a7)+,d0
         move.b     ($60001B),d6
         rts

wait_for_no_key                   ; don't clr.w d5 (else ESC+ON if low batt)
         movem.w    d5-d6,-(a7)
wait_for_no_key\loop
         move.w     #keyboard\all_rows_mask,d5  ; wait until all keys are
         bsr        get_mat                     ; released
         addq.b     #1,d6
         bne.s      wait_for_no_key\loop
         movem.w    (a7)+,d5-d6
         rts

get_key\wait_until_[on]_unpressed
         moveq.w    #10,d0
get_key\wait_until_[on]_unpressed\loop
         clr.l      apd\calls_5
         clr.b      apd\shut_down_if_idle
         btst.b     #1,$60001A
         beq        get_key\wait_until_[on]_unpressed\loop
         dbf.s      d0,get_key\wait_until_[on]_unpressed\loop
         rts

pause
         bsr        get_key
         tst.w      d6
         beq.s      pause
         rts

;                                keyboard.asm                                  ;
;============================================================================;