;============================================================================;
;                                  menu.asm                                  ;

menu_loop\start

         bsr        menu\open

menu_loop
         bsr        pause
	 pea        menu_loop(pc)

menu_loop\test_up_and_down
         bsr        menu\move

menu_loop\test_left_and_right
         cmp.w      #337,d6
         beq        menu\switch\right
         cmp.w      #340,d6
         beq        menu\switch\left

menu_loop\test_f1_to_f5
         cmp.w      #268,d6
         bcs.s      menu_loop\not_f1_to_f5
         cmp.w      #272,d6
         bhi.s      menu_loop\not_f1_to_f5
         bra        menu\open
menu_loop\not_f1_to_f5

menu_loop\test_1_to_9
         cmp.w      #49,d6
         bcs.s      menu_loop\not_1_to_9
         cmp.w      #57,d6
         bhi.s      menu_loop\not_1_to_9
         bra        menu\1_to_9_shortcut
menu_loop\not_1_to_9

         addq.l     #4,a7

menu_loop\test_enter
         cmp.w      #13,d6
         beq        menu\call_sub_routine

menu_loop\test_esc
         cmp.w      #264,d6
         beq.s      menu_loop\esc

menu_loop\test_shortcuts
         bra        test_shortcut

menu_loop\esc
         move.b     indicators_saved_value(pc),d0
         move.b     d0,indicators_value
         move.b     #1,indicators_clear_flag
         bra        menu\clear


;============================================================================;
;menu\open                                                                   ;
;    Input     : d6.w = F1 to F5 key value                                   ;
;    Output    : graphics                                                    ;
;    Destroyed : d0,d1,d2,d3,d4,a0                                              ;

menu\open
         move.w     d6,d0
         sub.w      #268,d0
         cmp.b      menu\number(pc),d0
         beq        menu\move\go_to_top
         move.b     d0,d6
         cmp.b      #$FF,menu\number
         beq.s      menu\open\new
         bsr        menu\clear
         bra.s      menu\open\show
menu\open\new
         move.b     indicators_value(pc),d0
         move.b     d0,indicators_saved_value
         clr.b      indicators_value
         bsr        refresh_indicators
menu\open\show
         move.b     d6,menu\number
         bsr        save_all_screen
         bra        menu\show

;============================================================================;
;menu\invert_title                                                           ;
;    Input     : menu\number.b                                               ;
;    Output    : graphics                                                    ;
;    Destroyed : d0,d1,d2,d3,d4                                              ;

menu\invert_title
         clr.w      d0
         move.b     menu\number(pc),d0
         mulu       #menu_title\width,d0
         addq.b     #1,d0
         moveq.w    #menu_corner\y+1,d1
         moveq.w    #menu_title\width-2,d2
         cmp.b      #4,menu\number
         bne.s      menu\invert_title\title_size
         subq.b     #1,d2               ; because 'help' is one pixel shorter
menu\invert_title\title_size
         moveq.w    #menu_title\height-1,d3
         clr.w      d4
         jsr        graphlib::fill          ; invert the menu title

 ifd ti92plus
         addq.w     #1,d0
         subq.w     #1,d1
         subq.w     #2,d2
         clr.w      d3
         jsr        graphlib::fill
 endc
         rts

;============================================================================;
;menu\show                                                                   ;
;    Input     : menu\number.b                                               ;
;    Output    : draws a menu                                                ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;


menu\show
         bsr        menu\invert_title
menu\show_2
         bsr        menu\frame
menu\show_3
         bsr        menu\show_text
         bra        menu\invert_item

menu\show_text
         cmp.b      #$FF,menu\number
         beq.s      menu\show\exit
         bsr        menu\set_parameters
         move.b     (a5)+,d0                ; menu X
         move.b     (a5)+,d1                ; menu Y
         move.b     (a5)+,d2                ; menu width
         move.b     (a5)+,d3                ; menu height
         move.b     (a5)+,d5                ; number of items
         lsl.b      #1,d5                   ; *2 (item + shortcut)
         addq.b     #1,d5                   ; because it's zeroed

         addq.b     #1,d0
         addq.b     #1,d1
         subq.b     #2,d2
         subq.b     #2,d3
	 moveq.w    #1,d4
	 jsr        graphlib::fill

         move.w     d0,d6
         addq.w     #1,d6
         subq.w     #5+menu_font*3,d1

menu\show_text\find_string
         tst.b      (a5)+
         bne.s      menu\show_text\find_string

         move.l     a5,a0
         move.b     d5,d4
         and.b      #1,d4
         beq.s      menu\show_text\set_shortcut

menu\show_text\set_item
         move.w     d6,d0
         add.w      #7+menu_font*3,d1
         bra.s      menu\show_text\draw

menu\show_text\set_shortcut
         move.w     #menu_font,d0
         bsr        set_font
         jsr        graphlib::getlength
         move.w     d6,d0
         add.w      d2,d0
         sub.w      d3,d0
         subq.w     #2,d0

menu\show_text\draw
         moveq.w    #menu_font,d3
         bsr        draw_string_a0
         dbf.s      d5,menu\show_text\find_string

menu\show\exit
         rts

menu\frame
         cmp.b      #$FF,menu\number
         beq.s      menu\frame\exit
         bsr        menu\set_parameters
         move.b     (a5),d0                 ; menu X
         move.b     1(a5),d1                ; menu Y
         move.b     2(a5),d4                ; menu width
         move.b     3(a5),d5                ; menu height
	 jsr        graphlib::frame
menu\frame\exit
         rts

;============================================================================;
;menu\clear                                                                  ;
;    Input     : menu\number.b                                               ;
;    Output    : void                                                        ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;

menu\clear
         cmp.b      #$FF,menu\number
         beq.s      menu\clear\exit
         bsr        restore_all_screen
         move.b     #$FF,menu\number
         clr.b      menu\item
menu\clear\exit
         rts


;============================================================================;
;menu\switch\right                                                           ;
;    Input     : void                                                        ;
;    Output    : switch to next menu                                         ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;

menu\switch\right
         move.b     menu\number,-(a7)
         bsr        menu\clear
         move.b     (a7)+,menu\number
         cmp.b      #4,menu\number
         bne.s      menu\switch\not_last
         move.b     #-1,menu\number
menu\switch\not_last
         addq.b     #1,menu\number
         bsr        save_all_screen
         bra        menu\show


;============================================================================;
;menu\switch\left                                                            ;
;    Input     : void                                                        ;
;    Output    : switch to next menu                                         ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;

menu\switch\left
         move.b     menu\number,-(a7)
         bsr        menu\clear
         move.b     (a7)+,menu\number
         tst.b      menu\number
         bne.s      menu\switch\not_first
         move.b     #5,menu\number
menu\switch\not_first
         subq.b     #1,menu\number
         bsr        save_all_screen
         bra        menu\show


;============================================================================;
;menu\set_parameters                                                         ;
;    Input     : menu\number.b                                               ;
;    Output    : a5                                                          ;
;    Destroyed : d0                                                          ;

menu\set_parameters
         clr.l      d0
         clr.w      d1
         clr.w      d2
         clr.w      d3
         clr.w      d4
         clr.w      d5
         lea        menu\list(pc),a5
         move.b     menu\number(pc),d0
         lsl.l      #2,d0
         move.l     0(a5,d0.l),a5       ; a5 points the menu
         rts

;============================================================================;
;menu\call_sub_routine                                                       ;
;    Input     : menu\number.b, menu\item.b                                  ;
;    Output    : executes the sub routine                                    ;
;    Destroyed : d6,d7,a0                                                    ;

menu\call_sub_routine
         clr.l      d6
         clr.l      d7
         move.b     menu\number(pc),d6
         move.b     menu\item(pc),d7
menu\call_sub_routine_2
         cmp.b      #$FF,menu\number
         beq.s      menu\call_sub_routine_2\no_menu_openned
         move.b     indicators_saved_value(pc),d0
         move.b     d0,indicators_value
         move.b     #1,indicators_clear_flag
         bsr        menu\clear
menu\call_sub_routine_2\no_menu_openned
         bsr        draw_info
         lsl.w      #2,d6
         lsl.w      #2,d7
         lea        menu\number_list(pc),a0
         movea.l    0(a0,d6.l),a0
         movea.l    0(a0,d7.l),a0
         jmp        (a0)

;============================================================================;
;menu\1_to_9_shortcut                                                        ;
;    Input     : d6 = [1] to [9] key value                                   ;
;    Output    : executes the sub routine                                    ;
;    Destroyed : d6,d7,a0                                                    ;

menu\1_to_9_shortcut
         sub.b      #49,d6
         bsr        menu\set_parameters
         cmp.b      4(a5),d6
         bhi.s      menu\1_to_9_shortcut\out_of_range
         bsr        menu\invert_item
         move.b     d6,menu\item
         bsr        menu\invert_item
         bsr        wait_for_no_key
         bra.s      menu\call_sub_routine
menu\1_to_9_shortcut\out_of_range
         rts


;============================================================================;
;menu\move                                                                   ;
;    Input     : d6.w = key value (up or down)                               ;
;    Output    : graphics                                                    ;
;    Destroyed : d0-d5/a5                                                    ;

menu\move
         move.l     (a7)+,a6
         bsr        menu\set_parameters
         cmp.w      #338,d6
         beq.s      menu\move\up
         cmp.w      #344,d6
         beq.s      menu\move\down
         and.w      #%1100111111111111,d6
         cmp.w      #338,d6
         beq.s      menu\move\go_to_top
         cmp.w      #344,d6
         beq.s      menu\move\go_to_bottom
         jmp        (a6)

menu\move\up
         tst.b      menu\item
         beq.s      menu\move\go_to_bottom
         bsr        menu\invert_item
         subq.b     #1,menu\item
         bra.s      menu\invert_item

menu\move\down
         move.b     4(a5),d0
         cmp.b      menu\item,d0
         beq.s      menu\move\go_to_top
         bsr        menu\invert_item
         addq.b     #1,menu\item
         bra.s      menu\invert_item

menu\move\go_to_top
         bsr        menu\invert_item
         clr.b      menu\item
         bra.s      menu\invert_item

menu\move\go_to_bottom
         bsr        menu\invert_item
         move.b     4(a5),menu\item

;============================================================================;
;menu\invert_item                                                            ;
;    Input     : menu\item.b                                                 ;
;    Output    : graphics                                                    ;
;    Destroyed : d0,d1,d2,d3,d4,a5                                           ;


menu\invert_item
         bsr        menu\set_parameters
         move.b     (a5),d0             ; menu X
         move.b     1(a5),d1            ; menu Y
         addq.b     #2,d0
         addq.b     #2,d1
         clr.w      d4
         move.b     menu\item(pc),d4
         mulu       #7+menu_font*3,d4
         add.b      d4,d1
         move.b     2(a5),d2
         subq.b     #4,d2
         moveq.w    #6+menu_font*3,d3
         clr.b      d4
         jmp        graphlib::fill


;                                  menu.asm                                  ;
;============================================================================;