;============================================================================;
;                                  vat.asm                                   ;
;============================================================================;
;get_handle                                                                  ;
;    Input     : d1.w = file or folder index of tex                          ;
;    Output    : d0.w = handle                                               ;
;    Destroyed : d1.l,a1-a2.l                                                ;
;============================================================================;
;get_index                                                                   ;
;    Input     : d1.w = file or folder index of tex                          ;
;    Output    : d1.w = file or folder index of tios                         ;
;    Destroyed : a1.l                                                        ;
;============================================================================;
;draw_name_details_and_info                                                  ;
;    Input     : d4.w = Y                                                    ;
;                d5.w = beginning index                                      ;
;                d6.w = number to draw                                       ;
;    Output    : void                                                        ;
;    Destroyed : d0.l/d1-d7.w,a0-a6.l                                        ;
;============================================================================;
;show_status_line                                                            ;
;    Input     : a0.l = address of string                                    ;
;    Output    : void                                                        ;
;    Destroyed : d0-d2.w,a0-a1.l                                             ;
;============================================================================;
;get_attributes                                                              ;
;    Input     : d1.w = file index                                           ;
;    Output    : a0.l = address of string with attributes                    ;
;    Destroyed : d0-d2.w,a1.l
;============================================================================;
;get_folder_size                                                             ;
;    Input     : void ( FILE LIST MUST BE ALLOCATED )                        ;
;    Output    : d0.l = folder size in bytes                                 ;
;    Destroyed : d0-d4.w,a1-a2.l                                             ;

;============================================================================;
;alloc_file_list                                                             ;
;    Description : Allocates enough RAM to fill a handle for the current     ;
;                  folder file list and saves it in handle\file_list\number  ;
;                  and handle\file_list\address (no HeapLock is done)        ;
;                  Adjusts number_files (which is the zeroed value of the    ;
;                  actual number of files) and  also sets                    ;
;                  number_files_to_draw which is min(number_files,10) so you ;
;                  know how many files you'll have to display                ;
;                  Clears cursor\file and current\file                       ;
;                  If enough memory can't be freed, alloc_file_list calls    ;
;                  errors\not_enough_memory                                  ;
;                  ALWAYS CALL THIS ROUTINE WHEN MODE_0 OR MODE_3            ;
;                  IS UNINITITIALIZED (USE UNINIT_MODE)                      ;
;    Input       : void                                                      ;
;    Output      : void                                                      ;
;    Destroyed   : d0-d3.w,a0-a3/a5.l                                        ;

alloc_file_list
         move.w     current\folder(pc),d1
         cmp.w      current\folder\old(pc),d1
         beq        alloc_file_list\same_folder
         clr.w      current\file
         clr.w      cursor\file
         move.w     d1,current\folder\old
alloc_file_list\same_folder
         bsr        get_handle\folder
         bsr        alloc_list
         move.w     d0,number_files_to_draw
         move.w     d1,number_files
         move.w     d2,handle\file_list\number
         move.l     a2,handle\file_list\address
         move.l     a3,sym_table\file
         rts

;============================================================================;
;alloc_folder_list                                                           ;
;    Description : Allocates enough RAM to fill a handle for the folder      ;
;                  list and saves it in handle\folder_list\number and        ;
;                  handle\folder_list\address (no HeapLock is done)          ;
;                  Adjusts number_folders (which is the zeroed value of      ;
;                  actual number of folders) and  also sets                  ;
;                  the number_folders_to_draw which is min(number_folders,10);
;                  so you know how many folders you'll have to display       ;
;                  Clears cursor\folder and current\folder                   ;
;                  If enough memory can't be freed, alloc_folder_list calls  ;
;                  errors\not_enough_memory                                  ;
;    Input       : void                                                      ;
;    Output      : void                                                      ;
;    Destroyed   : d0-d3.w,a0-a3/a5.l                                        ;


alloc_folder_list\clear_current
         clr.w      current\folder
         clr.w      cursor\folder
alloc_folder_list
         move.w     #tios::FolderListHandle,d0  ; folderlist
         bsr        alloc_list
         move.w     d0,number_folders_to_draw
         move.w     d1,number_folders
         move.w     d2,handle\folder_list\number
         move.l     a2,handle\folder_list\address
         move.l     a3,sym_table\folder
         rts

;============================================================================;
;alloc_list                                                                  ;
;    Input     : d0.w = file list or folder list handle                      ;                               ;
;    Output    : d0.w = number_to_draw                                       ;
;                d1.w = number                                               ;
;                d2.w = handle\number                                        ;
;                a2.l = handle\address                                       ;
;                a3.l = sym_table                                            ;
;    Destroyed :                                                             ;
;


;         move.w     #tios::FolderListHandle,d0
;         DEREF      d0,a2
;         addq.l     #2,a2
;         move.w     (a2)+,d3
;         move.w     d3,d4              ; d3 = number of folders
;
;         lsl.w      #1,d4
;         and.l      #FFFF,d4
;         move.l     d4,-(a7)
;         jsr        tios::HeapAllocHigh
;         tst.w      d0
;         beq        _exit              ; NE PAS OUBLIER not_enough_memory
;         move.w     d0,handle\folder_list\number
;         DEREF      d0,a3
;         move.l     a3,handle\folder_list\address
;
;         add.w      d4,2(a7)
;         jsr        tios::HeapAllocHigh
;         tst.w      d0
;         beq        _exit              ; NE PAS OUBLIER not_enough_memory
;         move.w     d0,handle\filelist_entry\number
;         DEREF      d0,a4
;         move.l     a4,handle\filelist_entry\address
;         move.l     a4,a5              ; saves a4
;
;         subq.w     #1,d3            ; for dbf
;         clr.w      d4               ; d4 = index counter
;         clr.w      d5               ; d5 = number of files counter
;
;alloc_lists\loop
;   *** FOLDER ATTRIBUTE ***
;         move.w     d4,(a3)+
;         move.w     12(a2),d0
;         DEREF      d0,a0
;         add.w      2(a0),d5
;         move.w     d4,(a3)+
;         move.l     a0,(a4)+
;         addq.w     #1,d4
;alloc_lists\loop\next
;         add.l      #14,a2
;         dbf.s      d3,alloc_lists\loop
;
;   *** D4 = NUMBER OF FOLDERS AFTER FILTER ***
;   *** D5 = NUMBER OF FILES BEFORE FILTER ***
;
;         move.w     d4,d0
;         addq.w     #1,d0
;         lsl.w      #1,d0
;         and.l      #FFFF,d0
;         move.l     d0,-(a7)
;         jsr        tios::HeapAllocHigh
;         tst.w      d0
;         beq        _exit              ; NE PAS OUBLIER not_enough_memory
;         move.w     d0,handle\first_file_list\number
;         DEREF      d0,a3
;         move.l     a3,handle\first_file_list\address
;
;         move.w     d5,d0
;         lsl.w      #1,d0
;         and.l      #FFFF,d0
;         move.l     d0,-(a7)
;         jsr        tios::HeapAllocHigh
;         tst.w      d0
;         beq        _exit              ; NE PAS OUBLIER not_enough_memory
;         move.w     d0,handle\file_list\number
;         DEREF      d0,a4
;         move.l     a4,handle\file_list\address
;
;         subq.w     #1,d4              ; zeroed
;         move.w     d4,d0
;         cmp.w      #number_drawed-1,d0
;         ble.s      alloc_lists\number_folder_to_draw
;         moveq.w    #number_drawed,d0
;
;alloc_lists\number_folder_to_draw
;         move.w     d0,number_folders_to_draw
;         move.w     d4,number_folders
;
;         cmp.w      #-1,d4
;         beq.s      alloc_lists\exit
;         tst.w      d5
;         beq.s      alloc_lists\exit
;
;         clr.w      d1               ; d1 = file counter
;         clr.w      (a3)+
;
;alloc_lists\folder_loop
;         clr.w      d2               ; d2 = file counter
;         move.l     (a5)+,a0
;         addq.l     #2,a0
;         move.w     (a0)+,d3
;         tst.w      d3
;         beq.s      alloc_lists\folder_loop\next
;         subq.w     #1,d3            ; for dbf
;
;alloc_lists\file_loop
;   *** FILE TYPE AND ATTRIBUTE ***
;         move.w     d2,(a4)+
;         addq.w     #1,d1
;         addq.w     #1,d2
;alloc_lists\file_loop\next
;         dbf.s      d3,alloc_lists\file_loop
;alloc_lists\folder_loop\next
;         move.w     d1,(a3)+
;         dbf.s      d4,alloc_lists\folder_loop
;
;         alloc_lists\exit
          rts

alloc_list
         move.w     d0,-(a7)           ; folder (/ folderlist) handle
         DEREF      d0,a3
         move.w     2(a3),d0
         move.w     d0,d3
         subq.w     #1,d3              ; zeroed value
         lsl.w      #1,d0
         and.l      #$FFFF,d0
         move.l     d0,-(a7)
         jsr        tios::HeapAllocHigh
         addq.l     #4,a7
         tst.w      d0
         beq        _exit               ; NE PAS OUBLIER not_enough_memory

         move.w     d0,d2               ; d2 = handle\number
         DEREF      d0,a0               ;
         move.l     a0,a2               ; a2 = handle\address
         move.l     a3,a1

         addq.l     #4,a1               ; points to first variable
         moveq.w    #-1,d1

         cmp.w      #-1,d3
         beq        alloc_list\exit     ; -> zero variable

         move.l     d2,-(a7)            ; handle\number
         move.w     d4,-(a7)

alloc_list\loop
         addq.w     #1,d1

         move.w     10(a1),d0                           ; attributes
         move.b     attribute_filter_flag(pc),d2        ; mask

test_temporary\starts_with_0
         cmp.b      #'0',(a1)                           ; starts with '0' ?
         beq.s      test_temporary\check_filter         ; yes,filter ?
         cmp.b      #'@',(a1)                           ; starts with '@' ?
         bne.s      test_protected                      ; no, next test
test_temporary\check_filter
         btst.b     #6,d2                               ; show it ?
         beq        alloc_list\loop\next                ; no, next...

test_protected
         tst.b      (a1)                                ; protected ?
         bne.s      test_archived                       ; no, next test
         btst.b     #4,attribute_filter_flag(pc)        ; yes, show it ?
         bne.s      protected\test_archived     ; yes, don't test hidden
         bra        alloc_list\loop\next        ; no, next file / folder

test_archived
         btst.w     #7,d0                       ; folder ?
         bne.s      test_locked                 ; yes, don't check RAM filter
         btst.w     #9,d0                       ; no, archived ?
         beq.s      unarchived                  ; no, check RAM filter
archived                                        ; yes, check archive filter
         btst.b     #1,d2                       ; show it ?
         beq.s      alloc_list\loop\next        ; no, forget that one
         bra.s      test_hidden                 ; yes, but don't test lock

unarchived
         btst.b     #0,d2                       ; show it ?
         beq.s      alloc_list\loop\next        ; no, forget it...

test_locked
         btst.w     #3,d0                       ; locked ?
         beq.s      test_hidden                 ; no, next test
         btst.b     #2,d2                       ; yes, show ?
         beq.s      alloc_list\loop\next        ; no, throw it

test_hidden
         btst.w     #4,d0                       ; hidden ?
         beq.s      test_type_and_zipped        ; no, next test
         btst.b     #3,d2                       ; yes, show ?
         beq.s      alloc_list\loop\next        ; no, warp it
         bra.s      test_type_and_zipped        ; yes, check type and zipped

protected\test_archived                         ; protected files are hidden
         btst.w     #7,d0                       ; folder ?
         bne.s      protected\test_locked       ; yes, don't check RAM filter
         btst.w     #9,d0                       ; no, archived ?
         beq.s      protected\unarchived        ; no, check RAM filter
protected\archived                              ; yes, check archive filter
         btst.b     #1,d2                       ; show it ?
         beq.s      alloc_list\loop\next        ; no, forget that one
         bra.s      test_type_and_zipped        ; yes, but don't test lock

protected\unarchived
         btst.b     #0,d2                       ; show it ?
         beq.s      alloc_list\loop\next        ; no, forget it...

protected\test_locked
         btst.w     #3,d0                       ; locked ?
         beq.s      test_type_and_zipped        ; no, next test
         btst.b     #2,d2                       ; yes, show ?
         beq.s      alloc_list\loop\next        ; no, throw it

test_type_and_zipped
         btst.b     #7,d0                       ; folder ?
         bne.s      alloc_list\loop\folder

alloc_list\loop\test_type
         move.w     6(a7),d0                    ; this was handle of folder
         jsr        filelib::gettype            ; d1.w = index of file
         clr.w      d0
         bset.w     d2,d0                       ; to check bit in mask
         and.w      type_filter_flag(pc),d0     ; show it ?
         beq.s      alloc_list\loop\next        ; no (bit clear)

alloc_list\loop\test_zip
         move.l     a0,d2                       ; saves a0
         move.l     a1,a0                       ; a0 = * SYMENTRY
         jsr        ziplib::iscomp              ; zipped (d0.w = 0) ?
         move.l     d2,a0                       ; restores a0
         tst.w      d0                          ; 0 = zipped, 1 = unzipped
         bne.s      alloc_list\loop\file        ; no, don't check mask

         btst.b     #5,attribute_filter_flag(pc)        ; yes, show it ?
         beq.s      alloc_list\loop\next        ; no, burn it

alloc_list\loop\file                    ; here everything is ok, add the file
alloc_list\loop\folder                  ; or folder to the list...
         move.w     d1,(a0)+            ; fill list with variable's indexes

alloc_list\loop\next                    ; end of loop, don't add to list
         add.l      #14,a1
         dbf.s      d3,alloc_list\loop

         move.w     (a7)+,d4
         move.l     (a7)+,d2            ; handle\number

         move.l     a0,d1
         sub.l      a2,d1
         lsr.l      #1,d1
         subq.w     #1,d1

         move.w     d1,d0
         cmp.w      #number_drawed-1,d0 ; d0 = number_to_draw = min(#9,number)
         ble.s      alloc_list\exit
         moveq.w    #number_drawed,d0

alloc_list\exit
         addq.l     #2,a7
         rts

;============================================================================;
unalloc_file_list
         cmp.w      #-1,number_folders
         beq        unalloc_file_list\no_folder
         move.w     handle\file_list\number(pc),-(a7)
         jsr        tios::HeapFree
         addq.l     #2,a7
unalloc_file_list\no_folder
         rts

;============================================================================;
unalloc_folder_list
         move.w     handle\folder_list\number(pc),-(a7)
         jsr        tios::HeapFree
         addq.l     #2,a7
         rts

;============================================================================;
;get_handle                                                                  ;
;    Input     : d1.w = file or folder index of tex                          ;
;    Output    : d0.w = handle                                               ;
;                d1.w = index                                                ;
;                a0.w = sym entry                                            ;
;    Destroyed : a1-a2.l                                                     ;

get_handle\folder\current
         move.w     current\folder(pc),d1

get_handle\folder
         move.l     sym_table\folder,a2
         bsr        get_index\folder
         bra.s      get_handle\with_index

get_handle\current
         move.w     current(pc),d1

get_handle
         move.l     sym_table(pc),a2
         bsr        get_index

get_handle\with_index
         move.w     d1,d0
         mulu       #14,d0
         lea        4(a2,d0.w),a2
         move.w     12(a2),d0
         rts

;============================================================================;
;get_index                                                                   ;
;    Input     : d1.w = file or folder index of tex                          ;
;    Output    : d1.w = file or folder index of tios                         ;
;    Destroyed : a1.l                                                        ;

get_index\folder\current
         move.w     current\folder(pc),d1

get_index\folder
         move.l     (handle\folder_list\address),a1
         bra.s      get_index\all

get_index\current
         move.w     current(pc),d1

get_index
         move.l     (handle\address),a1

get_index\all
         lsl.w      #1,d1
         move.w     0(a1,d1.w),d1
         rts

;============================================================================;
;draw_name_details_and_info                                                  ;
;    Input     : d4.w = Y                                                    ;
;                d5.w = beginning index                                      ;
;                d6.w = number to draw                                       ;
;    Output    : void                                                        ;
;    Destroyed : d0.l/d1-d7.w,a0-a6.l                                        ;

draw_name_and_details
         move.w     #4,-(a7)
         bra.s      draw_name_and_details\set_stack

draw_name_and_details\invert
         clr.w      -(a7)                     ; white on black

draw_name_and_details\set_stack
         lea        -4(a7),a7                     ; get space on the stack
         move.w     d4,-(a7)                      ; push Y
         move.w     window\name_absciss(pc),-(a7) ; push X
         move.w     d6,d7                         ; saves d6 (number to draw)

         move.l     (handle\address),a2
         move.l     sym_table(pc),a3
         move.w     d5,d0
         lsl.w      #1,d0
         add.w      d0,a2

draw_name\loop
         move.w     (a2)+,d0                  ; d0 = tios file index
         mulu       #14,d0
         lea        4(a3,d0.w),a4             ; a4 = sym entry
         move.b     8(a4),d3

draw_name\loop\test_selected
         btst.b     #5,11(a4)
         beq.s      draw_name\loop\test_protected
         move.w     (a7),d0
         subq.b     #7,d0
         move.w     2(a7),d1
         lea        sprites\check(pc),a0
         jsr        graphlib::put_sprite

draw_name\loop\test_protected
         tst.b      (a4)
         bne.s      draw_name\loop\draw_name
         lea        string,a1
         move.l     (a4)+,(a1)+
         move.l     (a4)+,(a1)+
         clr.b      (a1)
         lea        string,a4
         move.b     d3,(a4)

draw_name\loop\draw_name
         clr.b      8(a4)
         move.l     a4,4(a7)
         jsr        tios::DrawStrXY           ; draw name
         move.b     d3,8(a4)
         add.b      #8,3(a7)
         dbf.s      d7,draw_name\loop
         lea        10(a7),a7
                                              ; restore the stack
draw_details
         move.w     #4,-(a7)                  ; push color
         subq.l     #4,a7                     ; get space on the stack
         move.w     d4,-(a7)                  ; push Y
         subq.l     #2,a7                     ; get space on the stack

         tst.b      window
         beq        draw_details\window_0
         cmp.b      #1,window
         beq        draw_details\window_1
         cmp.b      #2,window
         beq.s      draw_details\window_2

draw_details\window_3
draw_details\window_3\loop
         bsr        unalloc_file_list
         move.w     d5,d1
         bsr        alloc_file_list\same_folder
         move.l     #1,d0                        ; convert d1 = number_files
         add.w      d1,d0                        ; to string (a0)
         bsr        d0_to_string                 ;
         move.l     a0,4(a7)                     ;
         move.w     window\type_absciss(pc),(a7) ;
         jsr        tios::DrawStrXY              ; draw number_files
         bsr        get_folder_size
         bsr        d0_to_string                   ; convert folder_size
         move.l     a0,4(a7)                       ; to string (a0)
         move.w     window\size_absciss(pc),(a7)   ;
         jsr        tios::DrawStrXY                ; draw  folder_size
         move.w     d5,d1
         bsr        get_attributes
         move.l     a0,4(a7)
         move.w     window\attr_absciss(pc),(a7)
         jsr        tios::DrawStrXY
         add.b      #8,3(a7)
         addq.w     #1,d5
         dbf.s      d6,draw_details\window_3\loop
         lea        10(a7),a7
         rts

draw_details\window_2
         move.w     d6,d7                         ; save d6
         move.w     d5,d3                         ; save d5
draw_details\window_2\loop
         move.w     d3,d1
         bsr        get_attributes
         move.l     a0,4(a7)
         move.w     window\attr_absciss(pc),(a7)
         jsr        tios::DrawStrXY
         add.b      #8,3(a7)
         addq.b     #1,d3
         dbf.s      d7,draw_details\window_2\loop
         move.b     d4,3(a7)

 ifd ti89
         bra.s      draw_details\window_1_and_2
 endc

draw_details\window_1

 ifd ti89
         clr.w      (a7)
         jsr        tios::FontSetSys               ; small font
         add.b      #2,3(a7)                       ; because of small font
 endc

draw_details\window_1_and_2
         bsr        get_handle\folder\current
         move.w     d0,d7                      ; d7 = current folder handle
         lea        variable_type(pc),a5       ; a5 points variable_types

draw_details\window_1_and_2\loop
         move.w     d5,d1                      ; current file index
         bsr        get_handle
         btst.w     #14,10(a2)
         beq.s      draw_details\window_1_and_2\loop\not_local
         cmp.w      #3,8(a2)
         beq.s      draw_details\window_1_and_2\loop\not_typed
draw_details\window_1_and_2\loop\not_local
         move.w     d7,d0                      ; current folder handle
         jsr        filelib::gettype
         cmp.b      #14,d2
         beq.s      draw_details\window_1_and_2\other_type
         move.b     d2,d0                    ;
         lsl.b      #2,d2                    ; mulu #5,d0
         add.b      d0,d2                    ;
         lea        0(a5,d2.w),a0            ; a0 points the variable type
         bra.s      draw_details\window_1_and_2\draw_type

draw_details\window_1_and_2\loop\not_typed
         lea        string\local(pc),a0
         bra.s      draw_details\window_1_and_2\draw_type

draw_details\window_1_and_2\other_type
         move.l     sym_table(pc),a2
         bsr        get_handle\with_index
         move.w     12(a2),d1
         DEREF      d1,a1
         clr.l      d1
         move.w     (a1),d1
         lea        string,a0
         move.b     -3(a1,d1.l),(a0)
         move.b     -2(a1,d1.l),1(a0)
         move.b     -1(a1,d1.l),2(a0)
         clr.b      3(a0)

draw_details\window_1_and_2\draw_type
         move.l     a0,4(a7)                     ;
         move.w     window\type_absciss(pc),(a7) ;
         jsr        tios::DrawStrXY              ; draw the variable type
         move.w     d5,d1                  ; uses d5 (beginning file index)
         bsr        get_handle             ; to get the file handle (a0)
         move.w     d0,d2
         clr.l      d0
         btst.w     #14,10(a2)
         beq.s      draw_details\window_1_and_2\not_local
         cmp.w      #3,8(a2)
         beq.s      draw_details\window_1_and_2\type_drawn
draw_details\window_1_and_2\not_local
         DEREF      d2,a0
         move.w     (a0),d0                      ; gets the file size
         addq.b     #2,d0                        ;
         bsr        d0_to_string                 ; convert file size to string
         move.l     a0,4(a7)                     ;
         move.w     window\size_absciss(pc),(a7) ;
         jsr        tios::DrawStrXY              ; draw the file size
draw_details\window_1_and_2\type_drawn
         add.b      #8,3(a7)
         addq.w     #1,d5
         dbf.s      d6,draw_details\window_1_and_2\loop
         move.w     #1,(a7)
         jsr        tios::FontSetSys             ; medium font

draw_details\window_0
         lea        10(a7),a7
         rts

draw_details\page
         move.w     #draw\y,d4
         move.w     current(pc),d5
         sub.w      cursor(pc),d5
         move.w     number_to_draw(pc),d6
         bra        draw_details

draw_split_window
         tst.b      window
         bne.s      draw_split_window\exit
         bsr        uninit_mode
         bsr        unalloc_file_list
         bsr        alloc_file_list
         bsr        init_mode\window_1
         cmp.w      #-1,number                         ; empty folder ?
         beq.s      draw_split_window\no_file
         move.w     current\file(pc),d5
         sub.w      cursor\file(pc),d5
         bsr        clear_window
         moveq.w    #draw\y,d4
         move.w     number_to_draw(pc),d6
         bsr        draw_name_and_details
draw_split_window\reinit_mode
         bra        init_mode\window_0

draw_split_window\no_file
         bsr        clear_window
         moveq.w    #window_1\name_absciss,d0
         moveq.w    #draw\y,d1
         moveq.w    #1,d3
         lea        string\no_file_found(pc),a0
         bsr        draw_string_a0
         bra.s      draw_split_window\reinit_mode

draw_split_window\exit
         rts

draw_info
         cmp.w      #-1,number_folders
         bne.s      draw_info\start
         rts

draw_info\start
         btst.b     #0,error_flag(pc)
         beq.s      draw_info\window

draw_info\error
         clr.b      error_flag
         bsr        show_status_line
         bra        wait_for_no_key

draw_info\window
         move.b     window(pc),d0
         addq.b     #1,d0
         and.b      #2,d0
         beq        draw_info\window_0_and_3

draw_info\window_1_and_2
         lea        string,a0
         clr.b      (a0)
         bsr        get_handle\folder\current
         move.w     d0,d3
         bsr        get_handle\current
         move.w     d3,d0
         btst.w     #14,10(a2)
         beq.s      draw_info\window_1_and_2\not_local
         cmp.w      #3,8(a2)
         beq        show_status_line

draw_info\window_1_and_2\not_local
         jsr        filelib::gettype
         bsr        get_handle\current
         move.w     d0,d3
         DEREF      d0,a1

draw_info\show_status_line\zipped_file
         move.w     (a1),d0              ; size
         lea        1(a1,d0.w),a2        ; type
         cmp.b      #$F8,(a2)                            ; zipped ?
         beq        show_status_line

draw_info\show_status_line\asm_file                      ; asm ?
	 cmp.l	    #"68kP",6(a1)
         bne.s      draw_info\show_status_line\text_file
     	 move.w     $A+2(a1),d0
         tst.w      d0
         beq        show_status_line
         lea        2(a1,d0.w),a0                        ; a0 points _comment
         bra        show_status_line

draw_info\show_status_line\text_file
         cmp.w      #7,d2                                ; text ?
         bne.s      draw_info\show_status_line\expr_list_mat_or_str
         lea        5(a1),a1
         move.l     a0,a2                            ; save a0
         moveq.w    #38,d0                           ; copy 39 chars
draw_info\show_status_line\text_file\loop            ; stop if end of line
         cmp.b      #13,(a1)
         beq.s      draw_info\show_status_line\text_file\loop\exit
         move.b     (a1)+,(a0)+
         dbf.s      d0,draw_info\show_status_line\text_file\loop
draw_info\show_status_line\text_file\loop\exit
         clr.b      (a0)
         move.l     a2,a0
         bra        show_status_line

draw_info\show_status_line\expr_list_mat_or_str
         move.w     d3,d0
    cmp.w      #4,d2
    beq.s viewlib\ExprToString
    cmp.w      #5,d2
    beq.s viewlib\ExprToString
    cmp.w      #8,d2
    beq.s viewlib\ExprToString
    cmp.w      #13,d2
    beq.s viewlib\ExprToString
    bra        show_status_line

; NEED TO USE VIEWLIB
viewlib\ExprToString
        clr.b   (string)                ; sinon a marche pas !?!?
	movem.l	d0-d7/a1-a6,-(a7)
	movea.l	a7,a6
	move.w	d0,-(a7)
	jsr	tios::HToESI
	move.b	(a0),-(a7)
	jsr	tios::QSysProtected
	subq.w	#$1,d0
	beq.s	viewlib\ExprToString\exit
	move.w	#LCD_WIDTH,d0
	lsr.w	#$2,d0
	move.w	d0,-(a7)
	clr.w	-(a7)
	move.l	a0,-(a7)
	jsr	tios::Parse1DExpr
	move.w	d0,-(a7)
	beq.s	viewlib\ExprToString\exit
	jsr	tios::HLock
        move.l  #39,-(a7)
        move.l  a0,-(a7)
        pea     (string)
        jsr     tios::strncpy
        lea     12(a7),a7
	jsr	tios::HeapFree
;        move.l  #string,a0
;        clr.b   39(a0)
;        lea     string,a0
        lea     (string),a0
        clr.b   39(a0)
viewlib\ExprToString\exit
	movea.l	a6,a7
	movem.l	(a7)+,d0-d7/a1-a6

         bra.s      show_status_line

draw_info\window_0_and_3
         bsr        uninit_mode
         bsr        unalloc_file_list
         bsr        alloc_file_list
         bsr        get_folder_size
         move.l     d0,-(a7)
         move.w     number_files(pc),-(a7)
         addq.w     #1,(a7)
         pea        string\files_bytes(pc)
         pea        string
         jsr        tios::sprintf
         lea        14(a7),a7
         lea        string,a0

;============================================================================;
;show_status_line                                                            ;
;    Input     : a0.l = address of string                                    ;
;    Output    : void                                                        ;
;    Destroyed : d0-d2.w,a0-a1.l                                             ;

show_status_line
         moveq.w    #status_line\x,d0
         moveq.w    #status_line\y,d1
         move.w     #status_line\width,d2
         moveq.w    #status_line\height,d3
         moveq.w    #1,d4
         jsr        graphlib::fill
         move.w     #4,-(a7)
         move.l     a0,-(a7)
         move.w     d1,-(a7)
         move.w     d0,-(a7)
         clr.w      -(a7)
         jsr        tios::FontSetSys
         addq.l     #2,a7
         jsr        tios::DrawStrXY
         move.w     #1,(a7)
         jsr        tios::FontSetSys
         lea        10(a7),a7
         bra        refresh_indicators

;============================================================================;
;get_attributes                                                              ;
;    Input     : d1.w = file index                                           ;
;    Output    : a0.l = address of string with attributes                    ;
;    Destroyed : d0-d2.w,a1-a2.l

get_attributes
         move.l     #string+1,a0
         bsr        get_handle
         move.w     10(a2),d0

         btst.b     #3,d0
         seq.b      d1
         and.b      #31,d1
         add.b      #14,d1
         move.b     d1,(a0)             ; locked

         btst.b     #4,d0
         seq.b      d1
         and.b      #10,d1
         add.b      #35,d1
         move.b     d1,1(a0)            ; hidden

         tst.b      (a2)
         seq.b      d1
         seq.b      d2
         and.b      #67,d1
         add.b      #45,d1
         move.b     d1,2(a0)            ; protected
         and.b      #10,d2
         add.b      d2,1(a0)            ; unhidden (when protected)

         clr.b      3(a0)
         btst.b     #7,d0
         bne.s      get_attributes\exit ; folder ?

         move.b     #'-',d1
         btst.b     #14,d0
         beq.s      get_attributes\not_local
         cmp.w      #3,8(a2)
         beq.s      get_attributes\local

get_attributes\not_local
         move.w     12(a2),d1
         DEREF      d1,a1
         clr.l      d1
         move.w     (a1),d1             ; size
         lea        -3(a1,d1.l),a1      ; type
         cmp.b      #'Z',(a1)+
         seq.b      d1
         cmp.b      #'I',(a1)+
         seq.b      d2
         and.b      d2,d1
         cmp.b      #'P',(a1)+
         seq.b      d2
         and.b      d2,d1
         cmp.b      #$F8,1(a1)
         seq.b      d2
         and.b      d2,d1
         and.b      #100,d1
         add.b      #45,d1
get_attributes\local
         move.b     d1,3(a0)            ; zipped

         btst.b     #9,d0
         sne.b      d0
         sne.b      d1
         and.b      #119,d1
         add.b      #45,d1
         move.b     d1,-(a0)            ; archived
         and.b      #31,d0
         add.b      d0,1(a0)            ; unlocked (when archived)
         clr.b      5(a0)

get_attributes\exit
         rts

;============================================================================;
;get_folder_size                                                             ;
;    Input     : void ( FILE LIST MUST BE ALLOCATED )                        ;
;    Output    : d0.l = folder size in bytes                                 ;
;    Destroyed : d0-d4.w,a1-a2.l                                             ;

get_folder_size
         clr.l      d2
         cmp.w      #-1,number_files
         beq.s      get_folder_size\exit
         move.w     number_files(pc),d3
         bsr        uninit_mode
         move.b     window,d4
         bsr        init_mode\window_1
get_folder_size\loop
         move.w     d3,d1
         bsr        get_handle
         btst.w     #14,10(a2)
         beq.s      get_folder_size\loop\not_local
         cmp.w      #3,8(a2)
         beq.s      get_folder_size\loop\dbra
get_folder_size\loop\not_local
         DEREF      d0,a1
         clr.l      d0
         move.w     (a1),d0
         add.l      d0,d2
         addq.l     #2,d2
get_folder_size\loop\dbra
         dbf.s      d3,get_folder_size\loop
         move.b     d4,window
         bsr        init_mode
get_folder_size\exit
         move.l     d2,d0
         rts

;                                  vat.asm                                   ;
;============================================================================;